/*
 * util.h -- General utility functions
 *
 * Copyright (C) 2004 Henrique M. Holschuh <hmh@debian.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef UTIL__H
#define UTIL__H

#include <unistd.h>
#include <stdint.h>

/* Computes elapsed time in microseconds */
extern uint64_t elapsed_time(struct timeval *start,
                              struct timeval *stop);

typedef enum {
	KERNEL_UNSUPPORTED,
	KERNEL_LINUX_24,
	KERNEL_LINUX_26
} kernel_mode_t;

/* Returns kernel support level */
extern  kernel_mode_t kernel_mode( void );

#endif /* UTIL__H */
